#include "tokamak-c-wrapper.h"

#ifdef _WIN32
 #include <windows.h>
 #ifdef TOKAMAK_COMPILE_DLL
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved) {
  switch (ul_reason_for_call) {
    case DLL_PROCESS_ATTACH:
    case DLL_THREAD_ATTACH:
    case DLL_THREAD_DETACH:
    case DLL_PROCESS_DETACH:
      break;
  }
  return TRUE;
}
 #endif
#else

#endif

#ifdef __cplusplus
extern "C" {
#endif


/**************
* neSimulator *
**************/
neSimulator * CreateSimulator(const neSimulatorSizeInfo & sizeInfo, const neV3 & grav) {
  return neSimulator::CreateSimulator(sizeInfo, NULL, &grav);
}

void DestroySimulator(neSimulator * & sim){
  if (sim) {
    neSimulator::DestroySimulator(sim);
    sim=NULL;
  }
}

void SimulatorSetGravity(neSimulator * sim, f32 gx, f32 gy, f32 gz){
  neV3 g; g.Set(gx,gy,gz); sim->Gravity(g);
}
void SimulatorGetGravity(neSimulator * sim, f32 & gx, f32 & gy, f32 & gz){
  neV3 g = sim->Gravity();
  gx=g[0]; gy=g[1]; gz=g[2];
}

void SimulatorSetGravityV3(neSimulator * sim, const neV3 & g){
  sim->Gravity(g);
}
neV3 SimulatorGetGravityV3(neSimulator * sim){
  return sim->Gravity();
}


neRigidBody * SimulatorCreateRigidBody(neSimulator * sim){
  return sim->CreateRigidBody();
}
neRigidBody * SimulatorCreateRigidParticle(neSimulator * sim){
  return sim->CreateRigidParticle();
}
void SimulatorFreeRigidBody(neSimulator * sim, neRigidBody * & body){
  if (body) {
    sim->FreeRigidBody(body); body=NULL;
  }
}

neAnimatedBody * SimulatorCreateAnimatedBody(neSimulator * sim){
  return sim->CreateAnimatedBody();
}
void SimulatorFreeAnimatedBody(neSimulator * sim, neAnimatedBody * & body) {
  if (body) {
    sim->FreeAnimatedBody(body); body=NULL;
  }
}

bool SimulatorSetMaterial(neSimulator * sim, s32 index, f32 friction, f32 restitution){
  return sim->SetMaterial(index,friction,restitution);
}
bool SimulatorGetMaterial(neSimulator * sim, s32 index, f32& friction, f32& restitution){
  return sim->GetMaterial(index,friction,restitution);
}

void SimulatorAdvance(neSimulator * sim, f32 sec){
  sim->Advance(sec,1,0);
}
void SimulatorAdvanceSteps(neSimulator * sim, f32 sec, s32 steps){
  sim->Advance(sec,steps,0);
}
void SimulatorAdvanceMinMax(neSimulator * sim, f32 sec, f32 minTimeStep, f32 maxTimeStep){
  sim->Advance(sec,minTimeStep,maxTimeStep,0);
}

void SimulatorSetTerrainMesh(neSimulator * sim, neTriangleMesh * tris){
  sim->SetTerrainMesh(tris);
}
void SimulatorFreeTerrainMesh(neSimulator * sim){
  sim->FreeTerrainMesh();
}
neJoint * SimulatorCreateRigidBodyWorldJoint(neSimulator * sim, neRigidBody * bodyA){
  return sim->CreateJoint(bodyA);
}
neJoint * SimulatorCreateRigidBodyRigidBodyJoint(neSimulator * sim, neRigidBody * bodyA, neRigidBody * bodyB){
  return sim->CreateJoint(bodyA,bodyB);
}
neJoint * SimulatorCreateRigidBodyAnimatedBodyJoint(neSimulator * sim, neRigidBody * bodyA, neAnimatedBody * bodyB){
  return sim->CreateJoint(bodyA,bodyB);
}
void SimulatorFreeJoint(neSimulator * sim, neJoint * & joint){
  if(joint) {
    sim->FreeJoint(joint); joint=0;
  }
}

void SimulatorSetCollisionCallback(neSimulator * sim, neCollisionCallback * cb){
  sim->SetCollisionCallback(cb);
}
neCollisionCallback * SimulatorGetCollisionCallback(neSimulator * sim){
  return sim->GetCollisionCallback();
}

void SimulatorSetBreakageCallback(neSimulator * sim, neBreakageCallback * cb){
  sim->SetBreakageCallback(cb);
}
neBreakageCallback * SimulatorGetBreakageCallback(neSimulator * sim){
  return sim->GetBreakageCallback();
}

void SimulatorSetTerrainTriangleQueryCallback(neSimulator * sim, neTerrainTriangleQueryCallback * cb){
  sim->SetTerrainTriangleQueryCallback(cb);
}
neTerrainTriangleQueryCallback * SimulatorGetTerrainTriangleQueryCallback(neSimulator * sim){
  return sim->GetTerrainTriangleQueryCallback();
}

void SimulatorSetCustomCDRB2RBCallback(neSimulator * sim, neCustomCDRB2RBCallback * cb){
  sim->SetCustomCDRB2RBCallback(cb);
}
neCustomCDRB2RBCallback * SimulatorGetCustomCDRB2RBCallback(neSimulator * sim){
  return sim->GetCustomCDRB2RBCallback();
}

void SimulatorSetCustomCDRB2ABCallback(neSimulator * sim, neCustomCDRB2ABCallback * cb){
  sim->SetCustomCDRB2ABCallback(cb);
}
neCustomCDRB2ABCallback * SimulatorGetCustomCDRB2ABCallback(neSimulator * sim){
  return sim->GetCustomCDRB2ABCallback();
}

void SimulatorSetLogOutputCallback(neSimulator * sim, neLogOutputCallback * cb){
  sim->SetLogOutputCallback(cb);
}
neLogOutputCallback * SimulatorGetLogOutputCallback(neSimulator * sim){
  return sim->GetLogOutputCallback();
}

void SimulatorSetLogOutputLevel(neSimulator * sim, u32 lvl){
  sim->SetLogOutputLevel((neSimulator::LOG_OUTPUT_LEVEL)lvl);
}

neSimulatorSizeInfo SimulatorGetCurrentSizeInfo(neSimulator * sim){
  return sim->GetCurrentSizeInfo();
}
neSimulatorSizeInfo SimulatorGetStartSizeInfo(neSimulator * sim){
  return sim->GetStartSizeInfo();
}
void SimulatorGetMemoryAllocated(neSimulator * sim, s32 & memoryAllocated){
  sim->GetMemoryAllocated(memoryAllocated);
}
s32 SimulatorGetIdleBodyCount(neSimulator * sim){
  return sim->GetIdleBodyCount();
}

neCollisionTable * SimulatorGetCollisionTable(neSimulator * sim){
  return sim->GetCollisionTable();
}

/*******************
* neCollisionTable *
*******************/
void CollisionTableSet(neCollisionTable * tbl, s32 collisionID1, s32 collisionID2, u32 response){
  tbl->Set(collisionID1, collisionID2, (neCollisionTable::neReponseBitFlag)response);
}
u32 CollisionTableGet(neCollisionTable * tbl, s32 collisionID1, s32 collisionID2){
  return (u32)tbl->Get(collisionID1, collisionID2);
}
s32 CollisionTableGetMaxCollisionID(neCollisionTable * tbl){
  return tbl->GetMaxCollisionID();
}


/*************
* neGeometry *
*************/
void GeometrySetUserData(neGeometry * geo, void* userData){
  geo->SetUserData(userData);
}
void* GeometryGetUserData(neGeometry * geo){
  return geo->GetUserData();
}

void GeometrySetCubeSize(neGeometry * geo, f32 cubeSize){
  geo->SetBoxSize(cubeSize,cubeSize,cubeSize);
}
void GeometrySetBoxSize(neGeometry * geo, f32 width, f32 height, f32 depth){
  geo->SetBoxSize(width,height,depth);
}
bool GeometryGetBoxSize(neGeometry * geo, f32 & width, f32 & height, f32 & depth){
  neV3 boxSize;
  bool ok = (geo->GetBoxSize(boxSize)!=0);
  width=boxSize[0]; height=boxSize[1]; depth=boxSize[2];
  return ok;
}

void GeometrySetBoxSizeV3(neGeometry * geo, const neV3 & boxSize){
  geo->SetBoxSize(boxSize);
}
bool GeometryGetBoxSizeV3(neGeometry * geo, neV3 & boxSize){
  return (geo->GetBoxSize(boxSize)!=0);
}

void GeometrySetSphereDiameter(neGeometry * geo, f32 diameter){
  geo->SetSphereDiameter(diameter);
}
bool GeometryGetSphereDiameter(neGeometry * geo, f32 & diameter){
  return (geo->GetSphereDiameter(diameter)!=0);
}

void GeometrySetCapsule(neGeometry * geo, f32 diameter, f32 height){
  geo->SetCylinder(diameter,height);
}
bool GeometryGetCapsule(neGeometry * geo, f32 & diameter, f32 & height){
  return (geo->GetCylinder(diameter,height)!=0);
}

void GeometrySetConvexMesh(neGeometry * geo, neByte * convexData){
  geo->SetConvexMesh(convexData);
}
bool GeometryGetConvexMesh(neGeometry * geo, neByte * & convexData){
  return (geo->GetConvexMesh(convexData)!=0);
}

void GeometrySetPosition(neGeometry* geo, f32 x, f32 y, f32 z) {
  neT3 t = geo->GetTransform();  t.pos.Set(x,y,z); geo->SetTransform(t);
}
void GeometryGetPosition(neGeometry* geo, f32 & x, f32 & y, f32 & z) {
  neT3 t = geo->GetTransform(); x=t.pos[0]; y=t.pos[1]; z=t.pos[2];
}


void GeometrySetPositionRotation(neGeometry* geo, f32 px, f32 py, f32 pz, f32 rp, f32 ry, f32 rr) {
  neV3 v; neT3 t; t.pos.Set(px,py,pz); v.Set(NE_DEG_TO_RAD(rp), NE_DEG_TO_RAD(ry), NE_DEG_TO_RAD(rr));
  t.rot.RotateXYZ(v); geo->SetTransform(t);
}
void GeometryGetPositionRotation(neGeometry* geo, f32 & px, f32 & py, f32 & pz, f32 & rx, f32 & ry, f32 & rz){
  neT3 t = geo->GetTransform(); neV3 v = t.pos; neM3 m = t.rot;  px=v[0]; py=v[1]; pz=v[2];
  v=RotAngle(m);
  rx=NE_RAD_TO_DEG(v[0]); ry=NE_RAD_TO_DEG(v[1]); rz=NE_RAD_TO_DEG(v[2]);
}
void GeometrySetPositionRotationV3(neGeometry* geo, const neV3 & position, const neV3 & rotation) {
  neT3 t; neV3 v; v.Set(NE_DEG_TO_RAD(rotation[0]), NE_DEG_TO_RAD(rotation[1]), NE_DEG_TO_RAD(rotation[2]));
  t.pos=position;  t.rot.RotateXYZ(rotation); geo->SetTransform(t);
}
void GeometryGetPositionRotationV3(neGeometry* geo, neV3 & position, neV3 & rotation){
  neT3 t = geo->GetTransform(); position = t.pos; neM3 m = t.rot;
  neV3 v = RotAngle(m);
  position[0]=NE_RAD_TO_DEG(v[0]); position[1]=NE_RAD_TO_DEG(v[1]); position[2]=NE_RAD_TO_DEG(v[2]);
}


void GeometrySetTransform(neGeometry * geo, neT3 & t){
  geo->SetTransform(t);
}
neT3 GeometryGetTransform(neGeometry * geo){
  return geo->GetTransform();
}


void GeometrySetMaterialIndex(neGeometry * geo, s32 index){
  geo->SetMaterialIndex(index);
}
s32  GeometryGetMaterialIndex(neGeometry * geo){
  return geo->GetMaterialIndex();
}

void GeometrySetBreakageFlag(neGeometry * geo, u32 flag){
  geo->SetBreakageFlag(flag);
}
u32 GeometryGetBreakageFlag(neGeometry * geo){
  return geo->GetBreakageFlag();
}

void GeometrySetBreakageMass(neGeometry * geo, f32 mass){
  geo->SetBreakageMass(mass);
}
f32  GeometryGetBreakageMass(neGeometry * geo){
  return geo->GetBreakageMass();
}

void GeometrySetBreakageInertiaTensor(neGeometry * geo, const neV3 & tensor){
  geo->SetBreakageInertiaTensor(tensor);
}
neV3 GeometryGetBreakageInertiaTensor(neGeometry * geo){
  return geo->GetBreakageInertiaTensor();
}

void GeometrySetBreakageMagnitude(neGeometry * geo, f32 mag){
  geo->SetBreakageMagnitude(mag);
}
f32  GeometryGetBreakageMagnitude(neGeometry * geo){
  return geo->GetBreakageMagnitude();
}

void GeometrySetBreakageAbsorption(neGeometry * geo, f32 absorb){
  geo->SetBreakageAbsorption(absorb);
}
f32  GeometryGetBreakageAbsorption(neGeometry * geo){
  return geo->GetBreakageAbsorption();
}

void GeometrySetBreakagePlane(neGeometry * geo, const neV3 & planeNormal){
  geo->SetBreakagePlane(planeNormal);
}
neV3 GeometryGetBreakagePlane(neGeometry * geo){
  return geo->GetBreakagePlane();
}

void GeometrySetBreakageNeighbourRadius(neGeometry * geo, f32 radius){
  geo->SetBreakageNeighbourRadius(radius);
}
f32 GeometryGetBreakageNeighbourRadius(neGeometry * geo){
  return geo->GetBreakageNeighbourRadius();
}

void GeometryGetOpenGLMatrix(neGeometry* geo, f32 * p){
  neT3 t = geo->GetTransform();
  t.MakeD3DCompatibleMatrix();
  memcpy(p,&t,16*sizeof(f32));
}
void GeometryGetOpenGLMatrixInverted(neGeometry* geo, f32 * p){
  neT3 t = geo->GetTransform().FastInverse();
  t.MakeD3DCompatibleMatrix();
  memcpy(p,&t,16*sizeof(f32));
}

neV3 GeometryRotateVector(neGeometry* geo, const neV3 & v) {
  return geo->GetTransform().rot * v;
}
neV3 GeometryTransformVector(neGeometry* geo, const neV3 & v) {
  return geo->GetTransform() * v;
}
void GeometryRotateVectors(neGeometry* geo, neV3 * d, const neV3 * s, s32 n){
  s32 i; neT3 t = geo->GetTransform(); for (i=0;i<n;i++){ d[i] = t.rot * s[i]; }
}
void GeometryTransformVectors(neGeometry* geo, neV3 * d, const neV3 * s, s32 n){
  s32 i; neT3 t = geo->GetTransform(); for (i=0;i<n;i++) { d[i] = t * s[i];}
}

/*****************
* neAnimatedBody *
*****************/
void AnimatedBodySetUserData(neAnimatedBody * ab, void* userData){
  ab->SetUserData(userData);
}
void* AnimatedBodyGetUserData(neAnimatedBody * ab){
  return ab->GetUserData();
}

void AnimatedBodySetPosition(neAnimatedBody * ab, f32   x, f32   y, f32   z){
  neV3 v; v.Set(x,y,z); ab->SetPos(v);
}
void AnimatedBodyGetPosition(neAnimatedBody * ab, f32 & x, f32 & y, f32 & z){
  neT3 t = ab->GetTransform();
  neV3 v; v=t.pos;
  x=v[0]; y=v[1]; z=v[2];
}

void AnimatedBodySetPositionV3(neAnimatedBody * ab, const neV3 & p){
  ab->SetPos(p);
}
neV3 AnimatedBodyGetPositionV3(neAnimatedBody * ab){
  return ab->GetPos();
}

void AnimatedBodySetRotation(neAnimatedBody* ab, f32 p, f32 y, f32 r) {
  neV3 v; neT3 t;
  v.Set( NE_DEG_TO_RAD(p), NE_DEG_TO_RAD(y), NE_DEG_TO_RAD(r) );
  t.rot.RotateXYZ(v);
  ab->SetRotation(t.rot);
}
void AnimatedBodyGetRotation(neAnimatedBody* ab, f32 & p, f32 & y, f32 & r) {
  neV3 v; neM3 m;
  m = ab->GetRotationM3();
  v = RotAngle(m);
  p=NE_RAD_TO_DEG(v[0]);
  y=NE_RAD_TO_DEG(v[1]);
  r=NE_RAD_TO_DEG(v[2]);
}

void AnimatedBodySetRotationV3(neAnimatedBody* ab, const neV3 & rotation) {
  neT3 t; neV3 v; v.Set( NE_DEG_TO_RAD(rotation[0]), NE_DEG_TO_RAD(rotation[1]), NE_DEG_TO_RAD(rotation[2]) );
  t.rot.RotateXYZ(v);
  ab->SetRotation(t.rot);
}
neV3 AnimatedBodyGetRotationV3(neAnimatedBody* ab) {
  neV3 v; neM3 m; m = ab->GetRotationM3(); v = RotAngle(m);
  v[0]=NE_RAD_TO_DEG(v[0]);
  v[1]=NE_RAD_TO_DEG(v[1]);
  v[2]=NE_RAD_TO_DEG(v[2]);
  return v;
}

void AnimatedBodySetRotationM3(neAnimatedBody * ab, const neM3 & m){
  ab->SetRotation(m);
}
neM3 AnimatedBodyGetRotationM3(neAnimatedBody * ab){
  return ab->GetRotationM3();
}

void AnimatedBodySetRotationQ(neAnimatedBody * ab, const neQ & q){
  ab->SetRotation(q);
}
neQ AnimatedBodyGetRotationQ(neAnimatedBody * ab){
  return ab->GetRotationQ();
}

//! rotation in degree
void AnimatedBodySetPositionRotation(neAnimatedBody* ab, f32 px, f32 py, f32 pz, f32 rp, f32 ry, f32 rr) {
  neV3 v; neT3 t; v.Set(rp,ry,rr); t.rot.RotateXYZ(v); v.Set(NE_DEG_TO_RAD(px),NE_DEG_TO_RAD(py),NE_DEG_TO_RAD(pz));
  ab->SetPos(v); ab->SetRotation(t.rot);
}
void AnimatedBodyGetPositionRotation(neAnimatedBody* ab, f32 & px, f32 & py, f32 & pz, f32 & rx, f32 & ry, f32 & rz){
  neV3 v = ab->GetPos(); neM3 m = ab->GetRotationM3(); px=v[0]; py=v[1]; pz=v[2];  v=RotAngle(m);
  rx=NE_RAD_TO_DEG(v[0]); ry=NE_RAD_TO_DEG(v[1]); rz=NE_RAD_TO_DEG(v[2]);
}
void AnimatedBodySetPositionRotationV3(neAnimatedBody* ab, const neV3 & position, const neV3 & rotation) {
  neT3 t; neV3 v; v.Set(NE_DEG_TO_RAD(rotation[0]),NE_DEG_TO_RAD(rotation[1]),NE_DEG_TO_RAD(rotation[2]));
  t.rot.RotateXYZ(v);
  ab->SetPos(position);
  ab->SetRotation(t.rot);
}
void AnimatedBodyGetPositionRotationV3(neAnimatedBody* ab, neV3 & position, neV3 & rotation){
  position = ab->GetPos(); neM3 m = ab->GetRotationM3(); neV3 v = RotAngle(m);
  position[0]=NE_RAD_TO_DEG(v[0]); position[1]=NE_RAD_TO_DEG(v[1]); position[2]=NE_RAD_TO_DEG(v[2]);
}

void AnimatedBodySetTransform(neAnimatedBody * ab, const neT3 & t){
  ab->SetPos(t.pos); ab->SetRotation(t.rot);
}
neT3 AnimatedBodyGetTransform(neAnimatedBody * ab){
  return ab->GetTransform();
}

void AnimatedBodySetCollisionID(neAnimatedBody * ab, s32 cid){
  ab->SetCollisionID(cid);
}
s32 AnimatedBodyGetCollisionID(neAnimatedBody * ab){
  return ab->GetCollisionID();
}

s32 AnimatedBodyGetGeometryCount(neAnimatedBody * ab){
  return ab->GetGeometryCount();
}

void AnimatedBodyUpdateBoundingInfo(neAnimatedBody * ab){
  ab->UpdateBoundingInfo();
}

void AnimatedBodySetCollideConnected(neAnimatedBody * ab, bool yes){
  ab->CollideConnected(yes);
}
bool AnimatedBodyGetCollideConnected(neAnimatedBody * ab){
  return (ab->CollideConnected()!=0);
}

void AnimatedBodySetCollideDirectlyConnected(neAnimatedBody * ab, bool yes){
  ab->CollideDirectlyConnected(yes);
}
bool AnimatedBodyGetCollideDirectlyConnected(neAnimatedBody * ab){
  return (ab->CollideDirectlyConnected()!=0);
}

neGeometry * AnimatedBodyAddGeometry(neAnimatedBody * ab){
  return ab->AddGeometry();
}
bool AnimatedBodyRemoveGeometry(neAnimatedBody * ab, neGeometry * geo){
  return (ab->RemoveGeometry(geo)!=0);
}

void AnimatedBodyBeginIterateGeometry(neAnimatedBody * ab){
  ab->BeginIterateGeometry();
}
neGeometry * AnimatedBodyGetNextGeometry(neAnimatedBody * ab){
  return ab->GetNextGeometry();
}

neRigidBody * AnimatedBodyBreakGeometry(neAnimatedBody * ab, neGeometry * geo){
  return ab->BreakGeometry(geo);
}

void AnimatedBodySetUseCustomCollisionDetection(neAnimatedBody * ab, bool yes,  const neT3 * obb, f32 boundingRadius){
  ab->UseCustomCollisionDetection(yes,obb,boundingRadius);
}
bool AnimatedBodyGetUseCustomCollisionDetection(neAnimatedBody * ab){
  return (ab->UseCustomCollisionDetection()!=0);
}

neSensor * AnimatedBodyAddSensor(neAnimatedBody * ab){
  return ab->AddSensor();
}
bool AnimatedBodyRemoveSensor(neAnimatedBody * ab, neSensor * sen){
  return ab->RemoveSensor(sen);
}

void AnimatedBodyBeginIterateSensor(neAnimatedBody * ab){
  ab->BeginIterateSensor();
}
neSensor * AnimatedBodyGetNextSensor(neAnimatedBody * ab){
  return ab->GetNextSensor();
}

void AnimatedBodySetActive(neAnimatedBody * ab, bool yes){
  ab->Active(yes,(neRigidBody *)0);
}
bool AnimatedBodyGetActive(neAnimatedBody * ab){
  return (ab->Active()!=0);
}
void AnimatedBodySetActiveWithRigidBodyHint(neAnimatedBody * ab, bool yes, neRigidBody * hint){
  ab->Active(yes,hint);
}
void AnimatedBodySetActiveWithAnimatedBodyHint(neAnimatedBody * ab, bool yes, neAnimatedBody * hint){
  ab->Active(yes,hint);
}


void AnimatedBodyGetOpenGLMatrix(neAnimatedBody* ab, f32 * p) {
  neT3 t = ab->GetTransform();
  t.MakeD3DCompatibleMatrix();
  memcpy(p,&t,16*sizeof(f32));
}
void AnimatedBodyGetOpenGLMatrixInverted(neAnimatedBody* ab, f32 * p) {
  neT3 t = ab->GetTransform().FastInverse();
  t.MakeD3DCompatibleMatrix();
  memcpy(p,&t,16*sizeof(f32));
}

neV3 AnimatedBodyRotateVector(neAnimatedBody* ab, const neV3 & v) {
  neT3 t = ab->GetTransform();
  neV3 r = t.rot * v;
  return r;
}
neV3 AnimatedBodyTransformVector(neAnimatedBody* ab, const neV3 & v) {
  neT3 t = ab->GetTransform();
  neV3 r = t * v;
  return r;
}
void AnimatedBodyRotateVectors(neAnimatedBody* ab, neV3 * d, const neV3 * s, s32 n){
  s32 i; neT3 t = ab->GetTransform(); for (i=0;i<n;i++){ d[i] = t.rot * s[i]; }
}
void AnimatedBodyTransformVectors(neAnimatedBody* ab, neV3 * d, const neV3 * s, s32 n){
  s32 i; neT3 t = ab->GetTransform(); for (i=0;i<n;i++) { d[i] = t * s[i];}
}


/**************
* neRigidBody *
**************/
void RigidBodySetUserData(neRigidBody* rb, void* userData){
  rb->SetUserData(userData);
}
void* RigidBodyGetUserData(neRigidBody* rb){
  return rb->GetUserData();
}

void RigidBodySetMass(neRigidBody* rb, f32 mass){
  if (mass<1) mass=1.0f; /// !!! added:
  rb->SetMass(mass);
}
f32 RigidBodyGetMass(neRigidBody* rb){
  return rb->GetMass();
}

void RigidBodySetInertiaTensor(neRigidBody* rb, const neV3 & v){
  rb->SetInertiaTensor(v);
}
void RigidBodySetInertiaTensorM3(neRigidBody* rb, const neM3 & tensor){
  rb->SetInertiaTensor(tensor);
}

void RigidBodySetPosition(neRigidBody* rb, f32 x, f32 y, f32 z) {
  neV3 v; v.Set(x,y,z); rb->SetPos(v);
}
void RigidBodyGetPosition(neRigidBody* rb, f32 & p, f32 & y, f32 & r) {
  neV3 v; v = rb->GetPos();
  p=v[0]; y=v[1]; r=v[2];
}

void RigidBodySetPositionV3(neRigidBody* rb, const neV3 & p){
  rb->SetPos(p);
}
neV3 RigidBodyGetPositionV3(neRigidBody* rb){
  return rb->GetPos();
}

void RigidBodySetRotation(neRigidBody* rb, f32 p, f32 y, f32 r) {
  neV3 v; neM3 m;
  v.Set(NE_DEG_TO_RAD(p),NE_DEG_TO_RAD(y),NE_DEG_TO_RAD(r));
  m.RotateXYZ(v);
  rb->SetRotation(m);
}
void RigidBodyGetRotation(neRigidBody* rb, f32 & p, f32 & y, f32 & r) {
  neV3 v; neM3 m; m = rb->GetRotationM3(); v = RotAngle(m);
  p=NE_RAD_TO_DEG(v[0]);
  y=NE_RAD_TO_DEG(v[1]);
  r=NE_RAD_TO_DEG(v[2]);
}

void RigidBodySetRotationV3(neRigidBody* rb, const neV3 & rotation) {
  neT3 t; neV3 v;
  v.Set( NE_DEG_TO_RAD(rotation[0]),NE_DEG_TO_RAD(rotation[1]),NE_DEG_TO_RAD(rotation[2]) );
  t.rot.RotateXYZ(v); rb->SetRotation(t.rot);
}
neV3 RigidBodyGetRotationV3(neRigidBody* rb) {
  neV3 v; neM3 m; m = rb->GetRotationM3(); v = RotAngle(m);
  v[0]=NE_RAD_TO_DEG(v[0]);
  v[1]=NE_RAD_TO_DEG(v[1]);
  v[2]=NE_RAD_TO_DEG(v[2]);
  return v;
}

void RigidBodySetRotationM3(neRigidBody* rb, const neM3 & m){
  rb->SetRotation(m);
}
neM3 RigidBodyGetRotationM3(neRigidBody* rb){
  return rb->GetRotationM3();
}

void RigidBodySetRotationQ(neRigidBody* rb, const neQ & q){
  rb->SetRotation(q);
}
neQ  RigidBodyGetRotationQ(neRigidBody* rb){
  return rb->GetRotationQ();
}
//! rotation in degree
void RigidBodySetPositionRotation(neRigidBody* rb, f32 px, f32 py, f32 pz, f32 rp, f32 ry, f32 rr) {
  neV3 v; neT3 t; v.Set(NE_DEG_TO_RAD(rp),NE_DEG_TO_RAD(ry),NE_DEG_TO_RAD(rr));
  t.rot.RotateXYZ(v); v.Set(px,py,pz);
  rb->SetPos(v); rb->SetRotation(t.rot);
}
void RigidBodyGetPositionRotation(neRigidBody* rb, f32 & px, f32 & py, f32 & pz, f32 & rx, f32 & ry, f32 & rz){
  neV3 v = rb->GetPos(); neM3 m = rb->GetRotationM3();  px=v[0]; py=v[1]; pz=v[2];  v=RotAngle(m);
  rx=NE_RAD_TO_DEG(v[0]);  ry=NE_RAD_TO_DEG(v[1]); rz=NE_RAD_TO_DEG(v[2]);
}
void RigidBodySetPositionRotationV3(neRigidBody* rb, const neV3 & position, const neV3 & rotation) {
  neT3 t; neV3 v; v.Set(NE_DEG_TO_RAD(rotation[0]),NE_DEG_TO_RAD(rotation[1]),NE_DEG_TO_RAD(rotation[2]));
  t.rot.RotateXYZ(v); rb->SetPos(position); rb->SetRotation(t.rot);
}
void RigidBodyGetPositionRotationV3(neRigidBody* rb, neV3 & position, neV3 & rotation){
  position = rb->GetPos(); neM3 m = rb->GetRotationM3(); neV3 v = RotAngle(m);
  position[0]=NE_RAD_TO_DEG(v[0]); position[1]=NE_RAD_TO_DEG(v[1]); position[2]=NE_RAD_TO_DEG(v[2]);
}

neT3 RigidBodyGetTransform(neRigidBody* rb){
  return rb->GetTransform();
}

void RigidBodySetCollisionID(neRigidBody* rb, s32 cid){
  rb->SetCollisionID(cid);
}
s32 RigidBodyGetCollisionID(neRigidBody* rb){
  return rb->GetCollisionID();
}

s32 RigidBodyGetGeometryCount(neRigidBody* rb){
  return rb->GetGeometryCount();
}

void RigidBodySetLinearDamping(neRigidBody* rb, f32 damp){
  rb->SetLinearDamping(damp);
}
f32 RigidBodyGetLinearDamping(neRigidBody* rb){
  return rb->GetLinearDamping();
}

void RigidBodySetAngularDamping(neRigidBody* rb, f32 damp){
  rb->SetAngularDamping(damp);
}
f32 RigidBodyGetAngularDamping(neRigidBody* rb){
  return rb->GetAngularDamping();
}

void RigidBodySetSleepingParameter(neRigidBody* rb, f32 sleepParam){
  rb->SetSleepingParameter(sleepParam);
}
f32 RigidBodyGetSleepingParameter(neRigidBody* rb){
  return rb->GetSleepingParameter();
}

void RigidBodySetVelocity(neRigidBody* rb, const neV3 & v){
  rb->SetVelocity(v);
}
neV3 RigidBodyGetVelocity(neRigidBody* rb){
  return rb->GetVelocity();
}

neV3 RigidBodyGetVelocityAtPoint(neRigidBody* rb, const neV3 & pt){
  return rb->GetVelocityAtPoint(pt);
}

neV3 RigidBodyGetAngularVelocity(neRigidBody* rb){
  return rb->GetAngularVelocity();
}

void RigidBodySetAngularMomentum(neRigidBody* rb, const neV3& am){
  rb->SetAngularMomentum(am);
}
neV3 RigidBodyGetAngularMomentum(neRigidBody* rb){
  return rb->GetAngularMomentum();
}

void RigidBodyUpdateBoundingInfo(neRigidBody* rb){
  rb->UpdateBoundingInfo();
}

void RigidBodyUpdateInertiaTensor(neRigidBody* rb){
  rb->UpdateInertiaTensor();
}

void RigidBodySetForceAtPos(neRigidBody* rb, const neV3 & force, const neV3 & pos){
  rb->SetForce(force, pos);
}
void RigidBodySetForce(neRigidBody* rb, const neV3 & force){
  rb->SetForce(force);
}
neV3 RigidBodyGetForce(neRigidBody* rb){
  return rb->GetForce();
}

void RigidBodySetTorque(neRigidBody* rb, const neV3 & torque){
  rb->SetTorque(torque);
}
neV3 RigidBodyGetTorque(neRigidBody* rb){
  return rb->GetTorque();
}

void RigidBodyApplyImpulseAtPos(neRigidBody* rb, const neV3 & impulse, const neV3 & pos){
  rb->ApplyImpulse(impulse, pos);
}
void RigidBodyApplyImpulse(neRigidBody* rb, const neV3 & impulse){
  rb->ApplyImpulse(impulse);
}

void RigidBodyApplyTwist(neRigidBody* rb, const neV3 & twist){
  rb->ApplyTwist(twist);
}

void RigidBodyGravityEnable(neRigidBody* rb, bool yes){
  rb->GravityEnable(yes);
}
bool RigidBodyIsGravityEnable(neRigidBody* rb){
  return (rb->GravityEnable()!=0);
}

void RigidBodySetCollideConnected(neRigidBody* rb, bool yes){
  rb->CollideConnected(yes);
}
bool RigidBodyGetCollideConnected(neRigidBody* rb){
  return (rb->CollideConnected()!=0);
}

void RigidBodySetCollideDirectlyConnected(neRigidBody* rb, bool yes){
  rb->CollideDirectlyConnected(yes);
}
bool RigidBodyGetCollideDirectlyConnected(neRigidBody* rb){
  return (rb->CollideDirectlyConnected()!=0);
}

neGeometry * RigidBodyAddGeometry(neRigidBody* rb){
  return rb->AddGeometry();
}
bool RigidBodyRemoveGeometry(neRigidBody* rb, neGeometry * geo){
  return (rb->RemoveGeometry(geo)!=0);
}

void RigidBodyBeginIterateGeometry(neRigidBody* rb){
  rb->BeginIterateGeometry();
}
neGeometry * RigidBodyGetNextGeometry(neRigidBody* rb){
  return rb->GetNextGeometry();
}

neRigidBody * RigidBodyBreakGeometry(neRigidBody* rb, neGeometry * geo){
  return rb->BreakGeometry(geo);
}

void RigidBodySetUseCustomCollisionDetection(neRigidBody* rb, bool yes,  const neT3 * obb, f32 boundingRadius){
  rb->UseCustomCollisionDetection(yes,obb,boundingRadius);
}
bool RigidBodyGetUseCustomCollisionDetection(neRigidBody* rb){
  return (rb->UseCustomCollisionDetection()!=0);
}

neSensor * RigidBodyAddSensor(neRigidBody* rb){
  return rb->AddSensor();
}
bool RigidBodyRemoveSensor(neRigidBody* rb, neSensor * sen){
  return (rb->RemoveSensor(sen)!=0);
}

void RigidBodyBeginIterateSensor(neRigidBody* rb){
  rb->BeginIterateSensor();
}
neSensor * RigidBodyGetNextSensor(neRigidBody* rb){
  return rb->GetNextSensor();
}

void RigidBodySetActive(neRigidBody* rb, bool yes){
  rb->Active(yes,(neRigidBody *)0);
}
bool RigidBodyGetActive(neRigidBody* rb){
  return (rb->Active()!=0);
}
void RigidBodySetActiveWithRigidBodyHint(neRigidBody* rb, bool yes, neRigidBody * hint){
  rb->Active(yes, hint);
}
void RigidBodySetActiveWithAnimatedBodyHint(neRigidBody* rb, bool yes, neAnimatedBody * hint){
  rb->Active(yes, hint);
}


bool RigidBodyIsIdle(neRigidBody* rb){
  return (rb->IsIdle()!=0);
}

void RigidBodyGetOpenGLMatrix(neRigidBody* rb, f32 * p) {
  neT3 t = rb->GetTransform();
  t.MakeD3DCompatibleMatrix();
  memcpy(p,&t,16*sizeof(f32));
}
void RigidBodyGetOpenGLMatrixInverted(neRigidBody* rb, f32 * p) {
  neT3 t = rb->GetTransform().FastInverse();
  t.MakeD3DCompatibleMatrix();
  memcpy(p,&t,16*sizeof(f32));
}

neV3 RigidBodyRotateVector(neRigidBody* rb, const neV3 & v) {
  return rb->GetTransform().rot * v;
}
neV3 RigidBodyTransformVector(neRigidBody* rb, const neV3 & v) {
  return rb->GetTransform() * v;
}

void RigidBodyRotateVectors(neRigidBody* rb, neV3 * d, const neV3 * s, s32 n){
  s32 i; neT3 t = rb->GetTransform(); for (i=0;i<n;i++){ d[i] = t.rot * s[i]; }
}
void RigidBodyTransformVectors(neRigidBody* rb, neV3 * d, const neV3 * s, s32 n){
  s32 i; neT3 t = rb->GetTransform(); for (i=0;i<n;i++) { d[i] = t * s[i];}
}

/************************
* neRigidBodyController *
************************/
neRigidBodyController * RigidBodyAddController(neRigidBody* rb, RigidBodyControllerCallback * rbccb, s32 period){
  return rb->AddController(rbccb, period);
}
bool RigidBodyRemoveController(neRigidBody* rb, neRigidBodyController * controller){
  return (rb->RemoveController(controller)!=0);
}
void RigidBodyBeginIterateController(neRigidBody* rb){
  rb->BeginIterateController();
}
neRigidBodyController * RigidBodyGetNextController(neRigidBody* rb){
  return rb->GetNextController();
}

neRigidBody * RigidBodyControllerGetRigidBody(neRigidBodyController * controller){
  return controller->GetRigidBody();
}


void RigidBodyControllerSetForceWithTorque(neRigidBodyController * controller, f32 fx, f32 fy, f32 fz, f32 px, f32 py, f32 pz){
  neV3 force; neV3 position; force.Set(fx,fy,fz); position.Set(px,py,pz);
  controller->SetControllerForceWithTorque(force, position);
}
void RigidBodyControllerSetForce(neRigidBodyController * controller, f32 fx, f32 fy, f32 fz){
  neV3 force; force.Set(fx,fy,fz); controller->SetControllerForce(force);
}
void RigidBodyControllerGetForce(neRigidBodyController * controller, f32 & fx, f32 & fy, f32 &fz){
  neV3 force = controller->GetControllerForce(); fx=force[0]; fy=force[1]; fz=force[2];
}

void RigidBodyControllerSetTorque(neRigidBodyController * controller, f32 tx, f32 ty, f32 tz){
  neV3 torque; torque.Set(tx,ty,tz); controller->SetControllerTorque(torque);
}
void RigidBodyControllerGetTorque(neRigidBodyController * controller, f32 & tx, f32 & ty, f32 & tz){
  neV3 torque = controller->GetControllerTorque(); tx=torque[0]; ty=torque[1]; tz=torque[2];
}

void RigidBodyControllerSetForceWithTorqueV3(neRigidBodyController * controller, const neV3 & force, const neV3 & position){
  controller->SetControllerForceWithTorque(force, position);
}
void RigidBodyControllerSetForceV3(neRigidBodyController * controller, const neV3 & force){
  controller->SetControllerForce(force);
}
neV3 RigidBodyControllerGetForceV3(neRigidBodyController * controller){
  return controller->GetControllerForce();
}

void RigidBodyControllerSetTorqueV3(neRigidBodyController * controller, const neV3 & torque){
  controller->SetControllerTorque(torque);
}
neV3 RigidBodyControllerGetTorqueV3(neRigidBodyController * controller){
  return controller->GetControllerTorque();
}

/***********
* neSensor *
***********/
void SensorSetUserData(neSensor* sensor, void* userData){
  sensor->SetUserData(userData);
}
void* SensorGetUserData(neSensor* sensor){
  return sensor->GetUserData();
}
void SensorSetLineSensor(neSensor* sensor, f32 px,f32 py,f32 pz, f32 ex,f32 ey,f32 ez){
  neV3 position, lineVector; position.Set(px,py,pz); lineVector.Set(ex,ey,ez);
  sensor->SetLineSensor(position, lineVector);
}
void SensorSetLineSensorV3(neSensor* sensor, const neV3 & position, const neV3 & lineVector){
  sensor->SetLineSensor(position, lineVector);
}
void SensorGetLineVector(neSensor* sensor, f32 & x, f32 & y, f32 & z){
  neV3 v = sensor->GetLineVector();
  x=v[0]; y=v[1]; z=v[2];
}
neV3 SensorGetLineVectorV3(neSensor* sensor){
  return sensor->GetLineVector();
}

void SensorGetLineUnitVector(neSensor* sensor, f32 & x, f32 & y, f32 & z){
  neV3 v = sensor->GetLineUnitVector();
  x=v[0]; y=v[1]; z=v[2];
}
neV3 SensorGetLineUnitVectorV3(neSensor* sensor){
  return sensor->GetLineUnitVector();
}
void SensorGetLinePosition(neSensor* sensor, f32 & x, f32 & y, f32 & z){
  neV3 v = sensor->GetLinePos();
  x=v[0]; y=v[1]; z=v[2];
}
neV3 SensorGetLinePositionV3(neSensor* sensor){
  return sensor->GetLinePos();
}

f32 SensorGetDetectDepth(neSensor* sensor){
  return sensor->GetDetectDepth();
}

void SensorGetDetectNormal(neSensor* sensor, f32 & x, f32 & y, f32 & z){
  neV3 v = sensor->GetDetectNormal();
  x=v[0]; y=v[1]; z=v[2];
}
neV3 SensorGetDetectNormalV3(neSensor* sensor){
  return sensor->GetDetectNormal();
}

void SensorGetDetectContactPoint(neSensor* sensor, f32 & x, f32 & y, f32 & z){
  neV3 v = sensor->GetDetectContactPoint();
  x=v[0]; y=v[1]; z=v[2];
}
neV3 SensorGetDetectContactPointV3(neSensor* sensor){
  return sensor->GetDetectContactPoint();
}
neRigidBody * SensorGetDetectRigidBody(neSensor* sensor){
  return sensor->GetDetectRigidBody();
}
neAnimatedBody * SensorGetDetectAnimatedBody(neSensor* sensor){
  return sensor->GetDetectAnimatedBody();
}
s32 SensorGetDetectMaterial(neSensor* sensor){
  return sensor->GetDetectMaterial();
}

/**********
* neJoint *
**********/
void JointSetType(neJoint * joint, u32 t){
  joint->SetType((neJoint::ConstraintType)t);
}
u32 JointGetType(neJoint * joint){
  return (u32)joint->GetType();
}

void JointSetUserData(neJoint * joint, void* userData){
  joint->SetUserData(userData);
}
void* JointGetUserData(neJoint * joint){
  return joint->GetUserData();
}

neRigidBody * JointGetRigidBodyA(neJoint * joint){
  return joint->GetRigidBodyA();
}
neRigidBody * JointGetRigidBodyB(neJoint * joint){
  return joint->GetRigidBodyB();
}
neAnimatedBody * JointGetAnimatedBodyB(neJoint * joint){
  return joint->GetAnimatedBodyB();
}

void JointSetPositionRotationFrameA(neJoint * joint, f32 px,f32 py,f32 pz, f32 p,f32 y,f32 r){
  neT3 t; t.pos.v[0] = px; t.pos.v[1] = py; t.pos.v[2] = pz; t.pos.v[3] = 0;
  t.rot = RotMatrix( NE_DEG_TO_RAD(p), NE_DEG_TO_RAD(y), NE_DEG_TO_RAD(r) );
  joint->SetJointFrameA(t);
}
void JointSetPositionRotationFrameB(neJoint * joint, f32 px,f32 py,f32 pz, f32 p,f32 y,f32 r){
  neT3 t; t.pos.v[0] = px; t.pos.v[1] = py;  t.pos.v[2] = pz;  t.pos.v[3] = 0;
  t.rot = RotMatrix( NE_DEG_TO_RAD(p), NE_DEG_TO_RAD(y), NE_DEG_TO_RAD(r) ); joint->SetJointFrameB(t);
}
void JointSetPositionRotationFrameWorld(neJoint * joint, f32 px,f32 py,f32 pz, f32 p,f32 y,f32 r) {
  neT3 t; t.pos.v[0] = px; t.pos.v[1] = py; t.pos.v[2] = pz; t.pos.v[3] = 0;
  t.rot = RotMatrix( NE_DEG_TO_RAD(p), NE_DEG_TO_RAD(y), NE_DEG_TO_RAD(r) );
  joint->SetJointFrameWorld(t);
}
void JointSetPositionRotationV3FrameWorld(neJoint * joint, const neV3 & pos, const neV3 & rot) {
  neT3 t; t.pos = pos;
  t.rot = RotMatrix( NE_DEG_TO_RAD(rot[0]), NE_DEG_TO_RAD(rot[1]), NE_DEG_TO_RAD(rot[2]) );
  joint->SetJointFrameWorld(t);
}

void JointSetPositionRotationV3FrameA(neJoint * joint,  const neV3 & pos, const neV3 & rot) {
  neT3 t; t.pos=pos;
  t.rot = RotMatrix( NE_DEG_TO_RAD(rot[0]), NE_DEG_TO_RAD(rot[1]), NE_DEG_TO_RAD(rot[2]) );
  joint->SetJointFrameA(t);
}
void JointSetPositionRotationV3FrameB(neJoint * joint, const neV3 & pos, const neV3 & rot) {
  neT3 t; t.pos = pos;
  t.rot = RotMatrix( NE_DEG_TO_RAD(rot[0]), NE_DEG_TO_RAD(rot[1]), NE_DEG_TO_RAD(rot[2]) );
  joint->SetJointFrameB(t);
}


void JointSetFrameA(neJoint * joint, const neT3 & frameA) {
  joint->SetJointFrameA(frameA);
}
neT3 JointGetFrameA(neJoint * joint){
  return joint->GetJointFrameA();
}

void JointSetFrameB(neJoint * joint, const neT3 & frameB) {
  joint->SetJointFrameB(frameB);
}
neT3 JointGetFrameB(neJoint * joint){
  return joint->GetJointFrameB();
}

void JointSetFrameWorld(neJoint * joint, const neT3 & frame) {
  joint->SetJointFrameWorld(frame);
}

void JointSetJointLength(neJoint * joint, f32 length){
  joint->SetJointLength(length);
}
f32 JointGetJointLength(neJoint * joint){
  return joint->GetJointLength();
}

void JointEnable(neJoint * joint, bool yes){
  joint->Enable(yes);
}
bool JointIsEnabled(neJoint * joint){
  return (joint->Enable()!=0);
}

void JointSetDampingFactor(neJoint * joint, f32 factor){
  joint->SetDampingFactor(factor);
}
f32 JointGetDampingFactor(neJoint * joint){
  return joint->GetDampingFactor();
}

void JointSetEpsilon(neJoint * joint, f32 epsilon){
  joint->SetEpsilon(epsilon);
}
f32 JointGetEpsilon(neJoint * joint){
  return joint->GetEpsilon();
}

void JointSetIteration(neJoint * joint, s32 iteration){
  joint->SetIteration(iteration);
}
s32 JointGetIteration(neJoint * joint){
  return joint->GetIteration();
}

void JointSetUpperLimit(neJoint * joint, f32 limit){
  joint->SetUpperLimit(limit);
}
f32 JointGetUpperLimit(neJoint * joint){
  return joint->GetUpperLimit();
}

void JointSetLowerLimit(neJoint * joint, f32 limit){
  joint->SetLowerLimit(limit);
}
f32 JointGetLowerLimit(neJoint * joint){
  return joint->GetLowerLimit();
}

void JointLimitEnable(neJoint * joint, bool yes){
  joint->EnableLimit(yes);
}
bool JointIsLimitEnabled(neJoint * joint){
  return (joint->EnableLimit()!=0);
}

void JointSetUpperLimit2(neJoint * joint, f32 limit){
  joint->SetUpperLimit2(limit);
}
f32 JointGetUpperLimit2(neJoint * joint){
  return joint->GetUpperLimit2();
}

void JointSetLowerLimit2(neJoint * joint, f32 limit){
  joint->SetLowerLimit2(limit);
}
f32 JointGetLowerLimit2(neJoint * joint){
  return joint->GetLowerLimit2();
}

void JointLimit2Enable(neJoint * joint, bool yes){
  joint->EnableLimit2(yes);
}
bool JointIsLimit2Enabled(neJoint * joint){
  return (joint->EnableLimit2()!=0);
}

void JointMotorEnable(neJoint * joint, bool yes){
  joint->EnableMotor(yes);
}
bool JointIsMotorEnabled(neJoint * joint){
  return (joint->EnableMotor()!=0);
}

void JointSetMotor(neJoint * joint, f32 desireValue, f32 maxForce){
  joint->SetMotor(desireValue, maxForce);
}
void JointGetMotor(neJoint * joint, f32 & desireValue, f32 & maxForce){
 joint->GetMotor(desireValue, maxForce);
}

/********************
* neJointController *
********************/
neJointController * JointAddController(neJoint * joint, JointControllerCallback * jccb, s32 period){
  return joint->AddController(jccb, period);
}
bool JointRemoveController(neJoint * joint, neJointController * controller){
  return (joint->RemoveController(controller)!=0);
}
void JointBeginIterateController(neJoint * joint){
  joint->BeginIterateController();
}
neJointController * JointGetNextController(neJoint * joint){
  return joint->GetNextController();
}

neJoint * JointControllerGetJoint(neJointController * controller){
  return controller->GetJoint();
}

void JointControllerSetForceV3BodyA(neJointController * controller, const neV3 & force){
  controller->SetControllerForceBodyA(force);
}
neV3 JointControllerGetForceV3BodyA(neJointController * controller){
  return controller->GetControllerForceBodyA();
}

void JointControllerSetForceV3BodyB(neJointController * controller, const neV3 & force){
  controller->SetControllerForceBodyB(force);
}
neV3 JointControllerGetForceV3BodyB(neJointController * controller){
  return controller->GetControllerForceBodyB();
}

void JointControllerSetTorqueV3BodyA(neJointController * controller, const neV3 & torque){
  controller->SetControllerTorqueBodyA(torque);
}
neV3 JointControllerGetTorqueV3BodyA(neJointController * controller){
  return controller->GetControllerTorqueBodyA();
}

void JointControllerSetTorqueV3BodyB(neJointController * controller, const neV3 & torque){
  controller->SetControllerTorqueBodyB(torque);
}
neV3 JointControllerGetTorqueV3BodyB(neJointController * controller){
  return controller->GetControllerTorqueBodyB();
}

void JointControllerSetForceWithTorqueV3BodyA(neJointController * controller, const neV3 & force, const neV3 & pos){
  controller->SetControllerForceWithTorqueBodyA(force, pos);
}
void JointControllerSetForceWithTorqueV3BodyB(neJointController * controller, const neV3 & force, const neV3 & pos){
  controller->SetControllerForceWithTorqueBodyB(force, pos);
}

void FrameGetOpenGLMatrix(const neT3 & t,f32 * p) {
  if (p){
    neT3 frame = t;
    frame.MakeD3DCompatibleMatrix();
    memcpy(p,&frame,16*sizeof(f32));
  }
}
void FrameGetOpenGLMatrixInverted(neT3 & t, f32 * p) {
  if (p) {
    neT3 frame = t.FastInverse();
    frame.MakeD3DCompatibleMatrix();
    memcpy(p,&frame,16*sizeof(f32));
  }
}

neV3 FrameRotateVector(const neT3 & frame, const neV3 & v) {
  return frame.rot * v;
}
neV3 FrameTransformVector(neT3 & frame, const neV3 & v) {
  return frame * v;
}

void FrameRotateVectors(const neT3 & frame, neV3 * d, const neV3 * s, s32 n){
  if (n>0) {
    s32 i;
    for (i=0;i<n;i++){
      d[i] = frame.rot * s[i];
    }
  }
}
void FrameTransformVectors(neT3 & frame, neV3 * d, const neV3 * s, s32 n){
  if (n>0) {
    s32 i;
    for (i=0;i<n;i++) {
      d[i] = frame * s[i];
    }
  }
}



//! get pitch yaw roll from matrix
neV3 RotAngle(const neM3 & m3) {
  f32 gA[9];
  gA[0] = m3.M[0].v[0];
  gA[1] = m3.M[0].v[1];
  gA[2] = m3.M[0].v[2];
  //gA[3] = m3.M[1].v[0];
  gA[4] = m3.M[1].v[1];
  //gA[5] = m3.gA[1].v[2];
  gA[6] = m3.M[2].v[0];
  gA[7] = m3.M[2].v[1];
  gA[8] = m3.M[2].v[2];

  neV3 r;
  if (gA[7]<1.0f) {
    if (gA[7]>-1.0f) {
      r.v[0] = (f32)asin((double)gA[7]);
      r.v[1] = (f32)atan2(-gA[6],gA[8]);
      r.v[2] = (f32)atan2(-gA[1],gA[4]);
    } else {
      r.v[0] = -1.570796f;
      r.v[1] = 0.0f;
      r.v[2] = (f32)-atan2(gA[2],gA[0]);
    }
  } else {
    r.v[0] = 1.570796f;
    r.v[1] = 0.0f;
    r.v[2] = (f32)atan2(gA[2],gA[0]);
  }
  r.v[0] = NE_RAD_TO_DEG(r.v[0]);
  r.v[1] = NE_RAD_TO_DEG(r.v[1]);
  r.v[2] = NE_RAD_TO_DEG(r.v[2]);
  return r;
}
//! create matrix from pitch yaw roll vector
neM3 RotMatrixV3(const neV3 & A){
  neQ q;
  f32 p = NE_DEG_TO_RAD(A[0]); p*=0.5f;
  f32 y = NE_DEG_TO_RAD(A[1]); y*=0.5f;
  f32 r = NE_DEG_TO_RAD(A[2]); r*=0.5f;
  f32 cr = (f32)cos( r);
  f32 cp = (f32)cos( p);
  f32 cy = (f32)cos(-y);
  f32 sr = (f32)sin( r);
  f32 sp = (f32)sin( p);
  f32 sy = (f32)sin(-y);
  f32 cpcy = cp*cy;
  f32 spsy = sp*sy;
  f32 spcy = sp*cy;
  f32 cpsy = cp*sy;
  q.X = (cr*spcy)+(sr*cpsy);
  q.Y = (cr*cpsy)-(sr*spcy);
  q.Z = (sr*cpcy)-(cr*spsy);
  q.W = (cr*cpcy)+(sr*spsy);
  return q.BuildMatrix3();
}
//! create matrix from pitch yaw roll
neM3 RotMatrix(f32 p,f32 y,f32 r){
  neQ Rot;
  p=NE_DEG_TO_RAD(p); p*=0.5f;
  y=NE_DEG_TO_RAD(y); y*=0.5f;
  r=NE_DEG_TO_RAD(r); r*=0.5f;

  f32 cp = (f32)cos( p);
  f32 cy = (f32)cos(-y);
  f32 cr = (f32)cos( r);

  f32 sp = (f32)sin( p);
  f32 sy = (f32)sin(-y);
  f32 sr = (f32)sin( r);

  f32 cpcy = cp*cy;
  f32 spsy = sp*sy;
  f32 spcy = sp*cy;
  f32 cpsy = cp*sy;

  Rot.W = (cr*cpcy)+(sr*spsy);
  Rot.X = (cr*spcy)+(sr*cpsy);
  Rot.Y = (cr*cpsy)-(sr*spcy);
  Rot.Z = (sr*cpcy)-(cr*spsy);
  return Rot.BuildMatrix3();
}

void V3RemoveComponent(neV3 & v,neV3 o){
  v.RemoveComponent(o);
}

neV3 V3Lerp(const neV3 & a, const neV3 & b,f32 t){
  if (t<=0.0f)
    return a;
  else if (t>=1.0f)
    return b;
  return a + (b-a)*t;
}

void M3SetIdentiy(neM3 & m){
  m.SetIdentity();
}
neM3 M3FromQ(const neQ & q){
  return q.BuildMatrix3();
}
neM3 M3FromAxisAngle(const neV3 & axis, f32 angle){
  neQ q; q.Set(angle,axis); return q.BuildMatrix3();
}
neM3 M3FromEulerAngle(f32 p, f32 y, f32 r) {
  return RotMatrix(NE_DEG_TO_RAD(p),NE_DEG_TO_RAD(y),NE_DEG_TO_RAD(r));
}

neM3 M3AddM3(const neM3 & a, const neM3 & b) {
  return a + b;
}
neM3 M3Sub3(const neM3 & a, const neM3 & b) {
  return a - b;
}
neM3 M3Mul(const neM3 & a, f32 b) {
  return a * b;
}
neM3 M3MulM3(const neM3 & a, const neM3 & b) {
  return a * b;
}
neV3 M3MulV3(const neM3 & m, const neV3 v) {
  return m * v;
}


neM3 M3Lerp(const neM3 & ma, const neM3 & mb, f32 t){
  if (t<=0.0f)
    return ma;
  else if (t>=1.0f)
    return mb;
  else {
    neQ a;a.SetupFromMatrix3(ma);
    neQ b;b.SetupFromMatrix3(mb);
    neQ c = a + (b-a) * t;
    return c.BuildMatrix3();
  }
}

void T3SetIdentiy(neT3 & t){
  t.SetIdentity();
}
neV3 T3MulV3(neT3 & t, const neV3 & v){
  return t * v;
}
neT3 T3MulT3(neT3 & a, const neT3 & b){
  return a * b;
}
void T3Invers(neT3 & t) {
  t.FastInverse();
}
neT3 T3Inverted(const neT3 & t) {
  neT3 frame = t;
  frame.FastInverse();
  return frame;
}


void QNormalize(neQ & q){
  q.Normalize();
}
neQ QNormalized(neQ & q){
  return q.Normalize();
}
neQ QFromM3(const neM3 & m){
  neQ q;q.SetupFromMatrix3(m);
  return q;
}

neQ QMulQ(const neQ & a, const neQ & b){
  return a * b;
}

neQ QFromAxisAngle(const neV3 & axis, f32 angle){
  neQ q; q.Set(NE_DEG_TO_RAD(angle),axis);
  return q;
}
void QGetAxisAngle(const neQ & q, neV3 & axis, f32 & angle){
  q.GetAxisAngle(axis,angle);
  angle=NE_RAD_TO_DEG(angle);
}

neQ QLerp(const neQ & a, const neQ & b, f32 t){
  if (t<=0.0f)
    return a;
  else if (t>=1.0f)
    return b;
  else
    return a + (b-a) * t;
}



#ifdef __cplusplus
}
#endif
