#ifndef __tokamak_c_wrapper_h__
#define __tokamak_c_wrapper_h__

#include "math/ne_type.h"
#include "math/ne_debug.h"
#include "tokamak.h"
#include "containers.h"
#include "scenery.h"
#include "collision.h"
#include "constraint.h"
#include "rigidbody.h"
#include "stack.h"
#include "simulator.h"
#include "message.h"
// #include "Matrix3D.h"

#ifdef TOKAMAK_COMPILE_DLL
 #define CAPI __declspec(dllexport)
#else
 #define CAPI
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**************
* neSimulator *
***************/
neSimulator * CAPI CreateSimulator(const neSimulatorSizeInfo & sizeInfo, const neV3 & grav);
void CAPI DestroySimulator(neSimulator * & sim);

void CAPI SimulatorSetGravity(neSimulator * sim, f32 gx, f32 gy, f32 gz);
void CAPI SimulatorGetGravity(neSimulator * sim, f32 & gx, f32 & gy, f32 & gz);

void CAPI SimulatorSetGravityV3(neSimulator * sim, const neV3 & g);
neV3 CAPI SimulatorGetGravityV3(neSimulator * sim);

neRigidBody * CAPI SimulatorCreateRigidBody(neSimulator * sim);
neRigidBody * CAPI SimulatorCreateRigidParticle(neSimulator * sim);
void CAPI SimulatorFreeRigidBody(neSimulator * sim,neRigidBody * & body);

neAnimatedBody * CAPI SimulatorCreateAnimatedBody(neSimulator * sim);
void CAPI SimulatorFreeAnimatedBody(neSimulator * sim, neAnimatedBody * & body);

bool CAPI SimulatorSetMaterial(neSimulator * sim, s32 index, f32   friction, f32   restitution);
bool CAPI SimulatorGetMaterial(neSimulator * sim, s32 index, f32 & friction, f32 & restitution);

void CAPI SimulatorAdvance(neSimulator * sim, f32 sec);
void CAPI SimulatorAdvanceSteps(neSimulator * sim, f32 sec, s32 steps);
void CAPI SimulatorAdvanceMinMax(neSimulator * sim, f32 sec, f32 minTimeStep, f32 maxTimeStep);

void CAPI SimulatorSetTerrainMesh(neSimulator * sim, neTriangleMesh * tris);
void CAPI SimulatorFreeTerrainMesh(neSimulator * sim);

neJoint * CAPI SimulatorCreateRigidBodyWorldJoint(neSimulator * sim, neRigidBody * bodyA);
neJoint * CAPI SimulatorCreateRigidBodyRigidBodyJoint(neSimulator * sim, neRigidBody * bodyA, neRigidBody * bodyB);
neJoint * CAPI SimulatorCreateRigidBodyAnimatedBodyJoint(neSimulator * sim, neRigidBody * bodyA, neAnimatedBody * bodyB);
void CAPI SimulatorFreeJoint(neSimulator * sim, neJoint * & constraint);

void CAPI SimulatorSetCollisionCallback(neSimulator * sim, neCollisionCallback * fn);
neCollisionCallback * CAPI SimulatorGetCollisionCallback(neSimulator * sim);

void CAPI SimulatorSetBreakageCallback(neSimulator * sim, neBreakageCallback * cb);
neBreakageCallback * CAPI SimulatorGetBreakageCallback(neSimulator * sim);

void CAPI SimulatorSetTerrainTriangleQueryCallback(neSimulator * sim, neTerrainTriangleQueryCallback * cb);
neTerrainTriangleQueryCallback * CAPI SimulatorGetTerrainTriangleQueryCallback(neSimulator * sim);

void CAPI SimulatorSetCustomCDRB2RBCallback(neSimulator * sim, neCustomCDRB2RBCallback * cb);
neCustomCDRB2RBCallback * CAPI SimulatorGetCustomCDRB2RBCallback(neSimulator * sim);

void CAPI SimulatorSetCustomCDRB2ABCallback(neSimulator * sim, neCustomCDRB2ABCallback * cb);
neCustomCDRB2ABCallback * CAPI SimulatorGetCustomCDRB2ABCallback(neSimulator * sim);

void CAPI SimulatorSetLogOutputCallback(neSimulator * sim, neLogOutputCallback * fn);
neLogOutputCallback * CAPI SimulatorGetLogOutputCallback(neSimulator * sim);

void CAPI SimulatorSetLogOutputLevel(neSimulator * sim, u32 lvl);

neSimulatorSizeInfo CAPI SimulatorGetCurrentSizeInfo(neSimulator * sim);
neSimulatorSizeInfo CAPI SimulatorGetStartSizeInfo(neSimulator * sim);
void CAPI SimulatorGetMemoryAllocated(neSimulator * sim, s32 & memoryAllocated);
s32 CAPI SimulatorGetIdleBodyCount(neSimulator * sim);

neCollisionTable * CAPI SimulatorGetCollisionTable(neSimulator * sim);

/*******************
* neCollisionTable *
********************/
void CAPI CollisionTableSet(neCollisionTable * tbl, s32 collisionID1, s32 collisionID2, u32 response);
u32  CAPI CollisionTableGet(neCollisionTable * tbl, s32 collisionID1, s32 collisionID2);
s32  CAPI CollisionTableGetMaxCollisionID(neCollisionTable * tbl);

/*************
* neGeometry *
**************/
void CAPI GeometrySetUserData(neGeometry * geo, void* userData);
void* CAPI GeometryGetUserData(neGeometry * geo);

void CAPI GeometrySetCubeSize(neGeometry * geo, f32 cubeSize);

void CAPI GeometrySetBoxSize(neGeometry * geo, f32 width, f32 height, f32 depth);
bool CAPI GeometryGetBoxSize(neGeometry * geo, f32 & width, f32 & height, f32 & depth);

void CAPI GeometrySetBoxSizeV3(neGeometry * geo, const neV3 & boxSize);
bool CAPI GeometryGetBoxSizeV3(neGeometry * geo, neV3 & boxSize);

void CAPI GeometrySetCapsule(neGeometry * geo, f32   diameter, f32   height);
bool CAPI GeometryGetCapsule(neGeometry * geo, f32 & diameter, f32 & height);

void CAPI GeometrySetSphereDiameter(neGeometry * geo, f32   diameter);
bool CAPI GeometryGetSphereDiameter(neGeometry * geo, f32 & diameter);

void CAPI GeometrySetConvexMesh(neGeometry * geo, neByte *   convexData);
bool CAPI GeometryGetConvexMesh(neGeometry * geo, neByte * & convexData);

void CAPI GeometrySetPosition(neGeometry* geo, f32 x, f32 y, f32 z);
void CAPI GeometryGetPosition(neGeometry* geo, f32 & x, f32 & y, f32 & z);

void CAPI GeometrySetPositionRotation(neGeometry* geo, f32   px, f32   py, f32   pz, f32   rp, f32   ry, f32   rr);
void CAPI GeometryGetPositionRotation(neGeometry* geo, f32 & px, f32 & py, f32 & pz, f32 & rx, f32 & ry, f32 & rz);

void CAPI GeometrySetPositionRotationV3(neGeometry* geo, const neV3 & position, const neV3 & rotation);
void CAPI GeometryGetPositionRotationV3(neGeometry* geo,       neV3 & position,       neV3 & rotation);

void CAPI GeometrySetTransform(neGeometry * geo, neT3 & t);
neT3 CAPI GeometryGetTransform(neGeometry * geo);

void CAPI GeometrySetMaterialIndex(neGeometry * geo, s32 index);
s32  CAPI GeometryGetMaterialIndex(neGeometry * geo);

void CAPI GeometrySetBreakageFlag(neGeometry * geo, u32 flag);
u32  CAPI GeometryGetBreakageFlag(neGeometry * geo);

void CAPI GeometrySetBreakageMass(neGeometry * geo, f32 mass);
f32  CAPI GeometryGetBreakageMass(neGeometry * geo);

void CAPI GeometrySetBreakageInertiaTensor(neGeometry * geo, const neV3 & tensor);
neV3 CAPI GeometryGetBreakageInertiaTensor(neGeometry * geo);

void CAPI GeometrySetBreakageMagnitude(neGeometry * geo, f32 mag);
f32  CAPI GeometryGetBreakageMagnitude(neGeometry * geo);

void CAPI GeometrySetBreakageAbsorption(neGeometry * geo, f32 absorb);
f32  CAPI GeometryGetBreakageAbsorption(neGeometry * geo);

void CAPI GeometrySetBreakagePlane(neGeometry * geo, const neV3 & planeNormal);
neV3 CAPI GeometryGetBreakagePlane(neGeometry * geo);

void CAPI GeometrySetBreakageNeighbourRadius(neGeometry * geo, f32 radius);
f32  CAPI GeometryGetBreakageNeighbourRadius(neGeometry * geo);

void CAPI GeometryGetOpenGLMatrix(neGeometry* geo, f32 * p);
void CAPI GeometryGetOpenGLMatrixInverted(neGeometry* geo, f32 * p);
neV3 CAPI GeometryRotateVector(neGeometry* geo, const neV3 & v);
neV3 CAPI GeometryTransformVector(neGeometry* geo, const neV3 & v);
void CAPI GeometryRotateVectors(neGeometry* geo, neV3 * d, const neV3 * s, s32 n);
void CAPI GeometryTransformVectors(neGeometry* geo, neV3 * d, const neV3 * s, s32 n);

/*****************
* neAnimatedBody *
******************/
void   CAPI AnimatedBodySetUserData(neAnimatedBody * ab, void* userData);
void * CAPI AnimatedBodyGetUserData(neAnimatedBody * ab);

void CAPI AnimatedBodySetPosition(neAnimatedBody* ab, f32 x, f32 y, f32 z);
void CAPI AnimatedBodyGetPosition(neAnimatedBody* ab, f32 & x, f32 & y, f32 & z);

void CAPI AnimatedBodySetPositionV3(neAnimatedBody * ab, const neV3 & position);
neV3 CAPI AnimatedBodyGetPositionV3(neAnimatedBody * ab);

void CAPI AnimatedBodySetRotation(neAnimatedBody* ab, f32 p, f32 y, f32 r);
void CAPI AnimatedBodyGetRotation(neAnimatedBody* ab, f32 & p, f32 & y, f32 & r);

void CAPI AnimatedBodySetRotationV3(neAnimatedBody* ab, const neV3 & rotation);
neV3 CAPI AnimatedBodyGetRotationV3(neAnimatedBody* ab);

void CAPI AnimatedBodySetRotationM3(neAnimatedBody * ab, const neM3 & m);
neM3 CAPI AnimatedBodyGetRotationM3(neAnimatedBody * ab);

void CAPI AnimatedBodySetRotationQ(neAnimatedBody * ab, const neQ & q);
neQ  CAPI AnimatedBodyGetRotationQ(neAnimatedBody * ab);

void CAPI AnimatedBodySetPositionRotation(neAnimatedBody* ab, f32   px, f32   py, f32   pz, f32   rp, f32   ry, f32   rr);
void CAPI AnimatedBodyGetPositionRotation(neAnimatedBody* ab, f32 & px, f32 & py, f32 & pz, f32 & rx, f32 & ry, f32 & rz);

void CAPI AnimatedBodySetPositionRotationV3(neAnimatedBody* ab, const neV3 & position, const neV3 & rotation);
void CAPI AnimatedBodyGetPositionRotationV3(neAnimatedBody* ab,       neV3 & position,       neV3 & rotation);

void CAPI AnimatedBodySetTransform(neAnimatedBody * ab, const neT3 & t);
neT3 CAPI AnimatedBodyGetTransform(neAnimatedBody * ab);

void CAPI AnimatedBodySetCollisionID(neAnimatedBody * ab, s32 cid);
s32  CAPI AnimatedBodyGetCollisionID(neAnimatedBody * ab);

s32 CAPI AnimatedBodyGetGeometryCount(neAnimatedBody * ab);

void CAPI AnimatedBodyUpdateBoundingInfo(neAnimatedBody * ab);

void CAPI AnimatedBodySetCollideConnected(neAnimatedBody * ab, bool yes);
bool CAPI AnimatedBodyGetCollideConnected(neAnimatedBody * ab);

void CAPI AnimatedBodySetCollideDirectlyConnected(neAnimatedBody * ab, bool yes);
bool CAPI AnimatedBodyGetCollideDirectlyConnected(neAnimatedBody * ab);

neGeometry * CAPI AnimatedBodyAddGeometry(neAnimatedBody * ab);
bool CAPI AnimatedBodyRemoveGeometry(neAnimatedBody * ab, neGeometry * geo);
void CAPI AnimatedBodyBeginIterateGeometry(neAnimatedBody * ab);
neGeometry * CAPI AnimatedBodyGetNextGeometry(neAnimatedBody * ab);

neRigidBody * CAPI AnimatedBodyBreakGeometry(neAnimatedBody * ab, neGeometry * geo);

void CAPI AnimatedBodySetUseCustomCollisionDetection(neAnimatedBody * ab, bool yes,  const neT3 * obb, f32 boundingRadius);
bool CAPI AnimatedBodyGetUseCustomCollisionDetection(neAnimatedBody * ab);

neSensor * CAPI AnimatedBodyAddSensor(neAnimatedBody * ab);
bool CAPI AnimatedBodyRemoveSensor(neAnimatedBody * ab, neSensor * sen);

void CAPI AnimatedBodyBeginIterateSensor(neAnimatedBody * ab);
neSensor * CAPI AnimatedBodyGetNextSensor(neAnimatedBody * ab);

void CAPI AnimatedBodySetActive(neAnimatedBody * ab, bool yes);
bool CAPI AnimatedBodyGetActive(neAnimatedBody * ab);
void CAPI AnimatedBodySetActiveWithRigidBodyHint(neAnimatedBody * ab, bool yes, neRigidBody * hint);
void CAPI AnimatedBodySetActiveWithAnimatedBodyHint(neAnimatedBody * ab, bool yes, neAnimatedBody * hint);


void CAPI AnimatedBodyGetOpenGLMatrix(neAnimatedBody* ab, f32 * p);
void CAPI AnimatedBodyGetOpenGLMatrixInverted(neAnimatedBody* ab, f32 * p);
neV3 CAPI AnimatedBodyRotateVector(neAnimatedBody* ab, const neV3 & v);
neV3 CAPI AnimatedBodyTransformVector(neAnimatedBody* ab, const neV3 & v);
void CAPI AnimatedBodyRotateVectors(neAnimatedBody* ab, neV3 * d, const neV3 * s, s32 n);
void CAPI AnimatedBodyTransformVectors(neAnimatedBody* ab, neV3 * d, const neV3 * s, s32 n);

/**************
* neRigidBody *
***************/
void  CAPI RigidBodySetUserData(neRigidBody* rb, void* userData);
void* CAPI RigidBodyGetUserData(neRigidBody* rb);

void CAPI RigidBodySetMass(neRigidBody* rb, f32 mass);
f32  CAPI RigidBodyGetMass(neRigidBody* rb);

void CAPI RigidBodySetInertiaTensorM3(neRigidBody* rb, const neM3 & tensor);
void CAPI RigidBodySetInertiaTensor(neRigidBody* rb, const neV3 & v);

void CAPI RigidBodySetCollisionID(neRigidBody* rb, s32 cid);
s32 CAPI RigidBodyGetCollisionID(neRigidBody* rb);

s32 CAPI RigidBodyGetGeometryCount(neRigidBody* rb);

void CAPI RigidBodySetLinearDamping(neRigidBody* rb, f32 damp);
f32 CAPI RigidBodyGetLinearDamping(neRigidBody* rb);

void CAPI RigidBodySetAngularDamping(neRigidBody* rb, f32 damp);
f32 CAPI RigidBodyGetAngularDamping(neRigidBody* rb);

void CAPI RigidBodySetSleepingParameter(neRigidBody* rb, f32 sleepingParam);
f32 CAPI RigidBodyGetSleepingParameter(neRigidBody* rb);

void CAPI RigidBodySetPosition(neRigidBody* rb, f32   x, f32   y, f32   z);
void CAPI RigidBodyGetPosition(neRigidBody* rb, f32 & x, f32 & y, f32 & z);

void CAPI RigidBodySetPositionV3(neRigidBody* rb, const neV3 & p);
neV3 CAPI RigidBodyGetPositionV3(neRigidBody* rb);

void CAPI RigidBodySetRotation(neRigidBody* rb, f32   p, f32   y, f32   r);
void CAPI RigidBodyGetRotation(neRigidBody* rb, f32 & p, f32 & y, f32 & r);

void CAPI RigidBodySetRotationM3(neRigidBody* rb, const neM3 & m);
neM3 CAPI RigidBodyGetRotationM3(neRigidBody* rb);

void CAPI RigidBodySetRotationQ(neRigidBody* rb, const neQ & q);
neQ  CAPI RigidBodyGetRotationQ(neRigidBody* rb);

void CAPI RigidBodySetPositionRotation(neRigidBody* rb, f32   px, f32   py, f32   pz, f32   rp, f32   ry, f32   rr);
void CAPI RigidBodyGetPositionRotation(neRigidBody* rb, f32 & px, f32 & py, f32 & pz, f32 & rx, f32 & ry, f32 & rz);

void CAPI RigidBodySetPositionRotationV3(neRigidBody* rb, const neV3 & position, const neV3 & rotation);
void CAPI RigidBodyGetPositionRotationV3(neRigidBody* rb,       neV3 & position,       neV3 & rotation);

neT3 CAPI RigidBodyGetTransform(neRigidBody* rb);

void CAPI RigidBodySetVelocity(neRigidBody* rb, const neV3 & v);
neV3 CAPI RigidBodyGetVelocity(neRigidBody* rb);

neV3 CAPI RigidBodyGetVelocityAtPoint(neRigidBody* rb, const neV3 & pt);

neV3 CAPI RigidBodyGetAngularVelocity(neRigidBody* rb);

void CAPI RigidBodySetAngularMomentum(neRigidBody* rb, const neV3& am);
neV3 CAPI RigidBodyGetAngularMomentum(neRigidBody* rb);

void CAPI RigidBodyUpdateBoundingInfo(neRigidBody* rb);

void CAPI RigidBodyUpdateInertiaTensor(neRigidBody* rb);

void CAPI RigidBodySetForceAtPos(neRigidBody* rb, const neV3 & force, const neV3 & pos);
void CAPI RigidBodySetForce(neRigidBody* rb, const neV3 & force);
neV3 CAPI RigidBodyGetForce(neRigidBody* rb);

void CAPI RigidBodySetTorque(neRigidBody* rb, const neV3 & torque);
neV3 CAPI RigidBodyGetTorque(neRigidBody* rb);

void CAPI RigidBodyApplyImpulseAtPos(neRigidBody* rb, const neV3 & impulse, const neV3 & pos);
void CAPI RigidBodyApplyImpulse(neRigidBody* rb, const neV3 & impulse);

void CAPI RigidBodyApplyTwist(neRigidBody* rb, const neV3 & twist);

void CAPI RigidBodyGravityEnable(neRigidBody* rb, bool yes);
bool CAPI RigidBodyIsGravityEnable(neRigidBody* rb);

void CAPI RigidBodySetCollideConnected(neRigidBody* rb, bool yes);
bool CAPI RigidBodyGetCollideConnected(neRigidBody* rb);

void CAPI RigidBodySetCollideDirectlyConnected(neRigidBody* rb, bool yes);
bool CAPI RigidBodyGetCollideDirectlyConnected(neRigidBody* rb);

neGeometry * CAPI RigidBodyAddGeometry(neRigidBody* rb);
bool CAPI RigidBodyRemoveGeometry(neRigidBody* rb, neGeometry * geo);

void CAPI RigidBodyBeginIterateGeometry(neRigidBody* rb);
neGeometry * CAPI RigidBodyGetNextGeometry(neRigidBody* rb);

neRigidBody * CAPI RigidBodyBreakGeometry(neRigidBody* rb, neGeometry * geo);

void CAPI RigidBodySetUseCustomCollisionDetection(neRigidBody* rb, bool yes, const neT3 * obb, f32 boundingRadius);
bool CAPI RigidBodyGetUseCustomCollisionDetection(neRigidBody* rb);

neSensor * CAPI RigidBodyAddSensor(neRigidBody* rb);
bool CAPI RigidBodyRemoveSensor(neRigidBody* rb, neSensor * sen);

void CAPI RigidBodyBeginIterateSensor(neRigidBody* rb);
neSensor * CAPI RigidBodyGetNextSensor(neRigidBody* rb);

void CAPI RigidBodySetActive(neRigidBody* rb, bool yes);
bool CAPI RigidBodyGetActive(neRigidBody* rb);
void CAPI RigidBodySetActiveWithRigidBodyHint(neRigidBody* rb, bool yes, neRigidBody * hint);
void CAPI RigidBodySetActiveWithAnimatedBodyHint(neRigidBody* rb, bool yes, neAnimatedBody * hint);

bool CAPI RigidBodyIsIdle(neRigidBody* rb);

void CAPI RigidBodyGetOpenGLMatrix(neRigidBody* rb, f32 * p);
void CAPI RigidBodyGetOpenGLMatrixInverted(neRigidBody* rb, f32 * p);
neV3 CAPI RigidBodyRotateVector(neRigidBody* rb, const neV3 & v);
neV3 CAPI RigidBodyTransformVector(neRigidBody* rb, const neV3 & v);
void CAPI RigidBodyRotateVectors(neRigidBody* rb, neV3 * d, const neV3 * s, s32 n);
void CAPI RigidBodyTransformVectors(neRigidBody* rb, neV3 * d, const neV3 * s, s32 n);

/************************
* neRigidBodyController *
*************************/
// !!!! neRigidBodyController * CAPI RigidBodyAddController(neRigidBody* rb, neRigidBodyControllerCallback * controller, s32 period);
neRigidBodyController * CAPI RigidBodyAddController(neRigidBody* rb, RigidBodyControllerCallback * rbccb, s32 period);
bool CAPI RigidBodyRemoveController(neRigidBody* rb, neRigidBodyController * rbController);
void CAPI RigidBodyBeginIterateController(neRigidBody* rb);
neRigidBodyController * CAPI RigidBodyGetNextController(neRigidBody* rb);

neRigidBody * CAPI RigidBodyControllerGetRigidBody(neRigidBodyController * controller);

void CAPI RigidBodyControllerSetForceWithTorque(neRigidBodyController * controller, f32 fx, f32 fy, f32 fz, f32 px, f32 py, f32 pz);
void CAPI RigidBodyControllerSetForce(neRigidBodyController * controller, f32   fx, f32   fy, f32   fz);
void CAPI RigidBodyControllerGetForce(neRigidBodyController * controller, f32 & fx, f32 & fy, f32 & fz);

void CAPI RigidBodyControllerSetTorque(neRigidBodyController * controller, f32   tx, f32   ty, f32   tz);
void CAPI RigidBodyControllerGetTorque(neRigidBodyController * controller, f32 & tx, f32 & ty, f32 & tz);

void CAPI RigidBodyControllerSetForceWithTorqueV3(neRigidBodyController * controller, const neV3 & force, const neV3 & pos);
void CAPI RigidBodyControllerSetForceV3(neRigidBodyController * controller, const neV3 & force);
neV3 CAPI RigidBodyControllerGetForceV3(neRigidBodyController * controller);

void CAPI RigidBodyControllerSetTorqueV3(neRigidBodyController * controller, const neV3 & torque);
neV3 CAPI RigidBodyControllerGetTorqueV3(neRigidBodyController * controller);

/***********
* neSensor *
************/
void CAPI SensorSetUserData(neSensor* sensor, void* userData);
void* CAPI SensorGetUserData(neSensor* sensor);

void CAPI SensorSetLineSensor(neSensor* sensor, f32 px,f32 py,f32 pz, f32 ex,f32 ey,f32 ez);
void CAPI SensorGetLinePosition(neSensor* sensor, f32 & x, f32 & y, f32 & z);

void CAPI SensorSetLineSensorV3(neSensor* sensor, const neV3 & position, const neV3 & lineVector);
neV3 CAPI SensorGetLinePositionV3(neSensor* sensor);

neV3 CAPI SensorGetLineVectorV3(neSensor* sensor);
void CAPI SensorGetLineVector(neSensor* sensor, f32 & x, f32 & y, f32 & z);

neV3 CAPI SensorGetLineUnitVectorV3(neSensor* sensor);
void CAPI SensorGetLineUnitVector(neSensor* sensor, f32 & x, f32 & y, f32 & z);

f32 CAPI SensorGetDetectDepth(neSensor* sensor);

void CAPI SensorGetDetectNormal(neSensor* sensor, f32 & x, f32 & y, f32 & z);
neV3 CAPI SensorGetDetectNormalV3(neSensor* sensor);

void CAPI SensorGetDetectContactPoint(neSensor* sensor, f32 & x, f32 & y, f32 & z);
neV3 CAPI SensorGetDetectContactPointV3(neSensor* sensor);

neRigidBody * CAPI SensorGetDetectRigidBody(neSensor* sensor);

neAnimatedBody * CAPI SensorGetDetectAnimatedBody(neSensor* sensor);

s32 CAPI SensorGetDetectMaterial(neSensor* sensor);

/**********
* neJoint *
***********/
void CAPI JointSetType(neJoint * joint, u32 t);
u32 CAPI JointGetType(neJoint * joint);

void CAPI JointSetUserData(neJoint * joint, void* userData);
void* CAPI JointGetUserData(neJoint * joint);

neRigidBody * CAPI JointGetRigidBodyA(neJoint * joint);
neRigidBody * CAPI JointGetRigidBodyB(neJoint * joint);
neAnimatedBody * CAPI JointGetAnimatedBodyB(neJoint * joint);

void CAPI JointSetFrameA(neJoint * joint, const neT3 & frameA);
neT3 CAPI JointGetFrameA(neJoint * joint);

void CAPI JointSetFrameB(neJoint * joint, const neT3 & frameB);
neT3 CAPI JointGetFrameB(neJoint * joint);

void CAPI JointSetFrameWorld(neJoint * joint, const neT3 & frame);

void CAPI JointSetPositionRotationFrameA  (neJoint * joint, f32 px, f32 py, f32 pz, f32 rx, f32 ry, f32 rz);
void CAPI JointSetPositionRotationV3FrameA(neJoint * joint, const neV3 & position, const neV3 & rotation);

void CAPI JointSetPositionRotationFrameB  (neJoint * joint, f32 px, f32 py, f32 pz, f32 rx, f32 ry, f32 rz);
void CAPI JointSetPositionRotationV3FrameB(neJoint * joint, const neV3 & position, const neV3 & rotation);

void CAPI JointSetPositionRotationFrameWorld  (neJoint * joint, f32 px, f32 py, f32 pz, f32 rx, f32 ry, f32 rz);
void CAPI JointSetPositionRotationV3FrameWorld(neJoint * joint, const neV3 & position, const neV3 & rotation);

void CAPI JointSetJointLength(neJoint * joint, f32 length);
f32  CAPI JointGetJointLength(neJoint * joint);

void CAPI JointEnable(neJoint * joint, bool yes);
bool CAPI JointIsEnabled(neJoint * joint);

void CAPI JointSetDampingFactor(neJoint * joint, f32 factor);
f32  CAPI JointGetDampingFactor(neJoint * joint);

void CAPI JointSetEpsilon(neJoint * joint, f32 epsilon);
f32  CAPI JointGetEpsilon(neJoint * joint);

void CAPI JointSetIteration(neJoint * joint, s32 i);
s32  CAPI JointGetIteration(neJoint * joint);

void CAPI JointSetUpperLimit(neJoint * joint, f32 limit);
f32  CAPI JointGetUpperLimit(neJoint * joint);

void CAPI JointSetLowerLimit(neJoint * joint, f32 limit);
f32  CAPI JointGetLowerLimit(neJoint * joint);

void CAPI JointLimitEnable(neJoint * joint, bool yes);
bool CAPI JointIsLimitEnabled(neJoint * joint);

void CAPI JointSetUpperLimit2(neJoint * joint, f32 limit);
f32  CAPI JointGetUpperLimit2(neJoint * joint);

void CAPI JointSetLowerLimit2(neJoint * joint, f32 limit);
f32  CAPI JointGetLowerLimit2(neJoint * joint);

void CAPI JointLimit2Enable(neJoint * joint, bool yes);
bool CAPI JointIsLimit2Enabled(neJoint * joint);

void CAPI JointMotorEnable(neJoint * joint, bool yes);
bool CAPI JointIsMotorEnabled(neJoint * joint);

void CAPI JointSetMotor(neJoint * joint, f32   desireValue, f32   maxForce);
void CAPI JointGetMotor(neJoint * joint, f32 & desireValue, f32 & maxForce);

/********************
* neJointController *
*********************/
neJointController * CAPI JointAddController(neJoint * joint, JointControllerCallback * jccb, s32 period);
bool CAPI JointRemoveController(neJoint * joint, neJointController * controller);
void CAPI JointBeginIterateController(neJoint * joint);
neJointController * CAPI JointGetNextController(neJoint * joint);

neJoint * CAPI JointControllerGetJoint(neJointController * controller);

void CAPI JointControllerSetForceV3BodyA(neJointController * controller, const neV3 & force);
neV3 CAPI JointControllerGetForceV3BodyA(neJointController * controller);

void CAPI JointControllerSetForceV3BodyB(neJointController * controller, const neV3 & force);
neV3 CAPI JointControllerGetForceV3BodyB(neJointController * controller);

void CAPI JointControllerSetTorqueV3BodyA(neJointController * controller, const neV3 & torque);
neV3 CAPI JointControllerGetTorqueV3BodyA(neJointController * controller);

void CAPI JointControllerSetTorqueV3BodyB(neJointController * controller, const neV3 & torque);
neV3 CAPI JointControllerGetTorqueV3BodyB(neJointController * controller);

void CAPI JointControllerSetForceWithTorqueV3BodyA(neJointController * controller, const neV3 & force, const neV3 & pos);
void CAPI JointControllerSetForceWithTorqueV3BodyB(neJointController * controller, const neV3 & force, const neV3 & pos);

//! get pitch yaw roll from matrix
neV3 CAPI RotAngle(const neM3 & m3);
//! create matrix from pitch yaw roll vector
neM3 CAPI RotMatrixV3(const neV3 & v);
//! create matrix from pitch yaw roll
neM3 CAPI RotMatrix(f32 p, f32 y, f32 r);


void CAPI V3RemoveComponent(neV3 & v,neV3 o);
neV3 CAPI V3Lerp(const neV3 & a, const neV3 & b,f32 t);

void CAPI M3SetIdentiy(neM3 & m);
neM3 CAPI M3FromQ(const neQ & q);
neM3 CAPI M3FromAxisAngle(const neV3 & axis, f32 angle);
neM3 CAPI M3FromEulerAngle(f32 p,f32 y,f32 r);

neM3 CAPI M3AddM3(const neM3 & a, const neM3 & b);
neM3 CAPI M3SubM3(const neM3 & a, const neM3 & b);
neM3 CAPI M3MulM3(const neM3 & a, const neM3 & b);
neM3 CAPI M3Mul(const neM3 & a, f32 b);
neV3 CAPI M3MulV3(const neM3 & m, const neV3 v);

neM3 CAPI M3Lerp(const neM3 & ma, const neM3 & mb,f32 t);


void CAPI T3SetIdentiy(neT3 & t);
neV3 CAPI T3MulV3(neT3 & t, const neV3 & v);
neT3 CAPI T3MulT3(neT3 & a, const neT3 & b);
void CAPI T3Invers(neT3 & t);
neT3 CAPI T3Inverted(const neT3 & t);

void CAPI QNormalize(neQ & q);
neQ CAPI QNormalized(neQ & q);
neQ CAPI QFromM3(const neM3 & m);
neQ CAPI QMulQ(const neQ & a, const neQ & b);
neQ CAPI QFromAxisAngle(const neV3 & axis, f32 angle);
void CAPI QGetAxisAngle(const neQ & q, neV3& axis, f32 & angle);
neQ CAPI QLerp(const neQ & a, const neQ & b,f32 t);

void CAPI FrameGetOpenGLMatrix(const neT3 & frame,f32 * p);
void CAPI FrameGetOpenGLMatrixInverted(neT3 & frame, f32 * p);
neV3 CAPI FrameRotateVector(const neT3 & frame, const neV3 & v);
neV3 CAPI FrameTransformVector(neT3 & frame, const neV3 & v);
void CAPI FrameRotateVectors(const neT3 & frame, neV3 * d, const neV3 * s, s32 n);
void CAPI FrameTransformVectors(neT3 & frame, neV3 * d, const neV3 * s, s32 n);

#ifdef __cplusplus
}
#endif

#endif //! __tokamak_c_wrapper_h__
